<?php
if ( !defined('ABSPATH') ) exit;

class Voice_Forms_Settings_Page 
{
    // Database field name map
    const BASIC_CONFIG_OPTION_NAMES = array(
        'license_key'           => 'vf_license_key',
        'mic_listening_timeout' => 'vf_mic_listening_timeout',
        'selected_language'     => 'vf_selected_language',
        'floating_mic'          => 'vf_floating_mic'
    );

    private $vf_license_key           = '';
    private $vf_mic_listening_timeout = null;
    private $vf_selected_language     = 'English';
    private $vf_floating_mic          = null;

    /**
     * Start up
     */
    public function __construct()
    {
        add_action( 'admin_menu', array( $this, 'vf_add_plugin_page' ) );
        add_action( 'admin_init', array( $this, 'vf_page_init' ) );

        // Register callback to hook into post create and update (License key) option action
        add_action( 'add_option_'.self::BASIC_CONFIG_OPTION_NAMES['license_key'], array( $this, 'vf_post_adding_license_key'), 10, 2 );
        add_action( 'update_option_'.self::BASIC_CONFIG_OPTION_NAMES['license_key'], array( $this, 'vf_post_update_license_key'), 10, 2 );
    }

    /**
     * Method as callback post to license key option creation in DB
     *
     * @param $option_name - string : Option name
     * @param $option_value - string : Option value
     */
    public function vf_post_adding_license_key( $option_name, $option_value)
    {
        try {
            Voice_Forms_Plugin::vf_get_api_key_from_license_key(trim($option_value), true);
        } catch (\Exception $ex) {
            // Do nothing for now
        }
    }

    /**
     * Method as callback post to license key option update in DB
     *
     * @param $old_value - string : Option value before update
     * @param $new_value - string : Updated Option value
     */
    public function vf_post_update_license_key( $old_value, $new_value)
    {
        try {
            $option_value = strip_tags(stripslashes($new_value));
            
            if ($old_value != trim($option_value)) {
                Voice_Forms_Plugin::vf_get_api_key_from_license_key(trim($option_value), true);
            }
        } catch (\Exception $ex) {
            // Do nothing for now
        }
    }

    /**
     * Add options page
     */
    public function vf_add_plugin_page()
    {
        // This page will be under "Settings"
        add_submenu_page(
            'options-general.php',// Parent menu as 'settings'
            'Voice Forms',
            'Voice Forms',
            'manage_options',
            'voice-forms-settings',// Slug for page
            array( $this, 'vf_settings_create_page')// View 
        );
    }

    /**
     * Options/Settings page callback to create view/html of settings page
     */
    public function vf_settings_create_page()
    {
        // For license key
        $this->vf_license_key = strip_tags(stripslashes(get_option(self::BASIC_CONFIG_OPTION_NAMES['license_key'], '')));
        $this->vf_license_key = !empty($this->vf_license_key) ? $this->vf_license_key : '';

        if (empty($this->vf_license_key)) { update_option('vf_api_system_key', ''); }

        // For Mic listening auto timeout
        $this->vf_mic_listening_timeout = strip_tags(stripslashes(get_option(self::BASIC_CONFIG_OPTION_NAMES['mic_listening_timeout'], null)));

        // if voice type is blank then always store voice type as male
        if (empty($this->vf_mic_listening_timeout) || $this->vf_mic_listening_timeout < 8) {
            update_option(self::BASIC_CONFIG_OPTION_NAMES['mic_listening_timeout'], 8);
            $this->vf_mic_listening_timeout = 8;
        } elseif ($this->vf_mic_listening_timeout > 20) {
            update_option(self::BASIC_CONFIG_OPTION_NAMES['mic_listening_timeout'], 20);
            $this->vf_mic_listening_timeout = 20;
        }

         // For language
        $this->vf_selected_language = strip_tags(stripslashes(get_option( 
            self::BASIC_CONFIG_OPTION_NAMES['selected_language'], 'English')));

        // For floating mic
        $this->vf_floating_mic = strip_tags(stripslashes(get_option( 
            self::BASIC_CONFIG_OPTION_NAMES['floating_mic'], null)));
?>
        <div class="wrap">
            <div id="vfavigationSettingsWrapper">
                <div id="vfavigationSettingsHeader" class="vf-row">
                    <div class="vf-setting-header-column-1"><br>
                        <span id="vfavigationSettingsPageHeading">Voice Forms Setup</span>
                    </div>
                    <div class="vf-setting-header-column-2">
                        <a title="Wordpress Plugin - speak2web" target="blank" href="https://speak2web.com/plugin/">
                            <img id="vfavigationSettingsPageHeaderLogo" 
                            src="<?php echo dirname(plugin_dir_url(__FILE__)).'/images/speak2web_logo.png'?>">
                        </a>
                    </div>
                </div>

                <form id="vfavigationBasicConfigForm" method="post" action="options.php">
                    <?php
                        // This prints out all hidden setting fields
                        settings_fields( 'vf-basic-config-settings-group' );
                        do_settings_sections( 'vf-settings' );

                        // To display errors
                        settings_errors('vf-settings', true, true);
                    ?>
                    <div id="vfavigationBasicConfigSection" class='vf-row vf-card'>
                        <div id="vfBasicConfHeaderSection" class="vf-setting-basic-config-column-1 vf-basic-config-section-title">
                            <table id="vfavigationBasicConfHeaderTable">
                                <tr>
                                    <th><h4><u><?php echo VF_LANGUAGE_LIBRARY['basicConfig']['basicConfiguration']; ?></u></h4></th>
                                </tr>
                            </table>
                        </div>

                        <div class="vf-setting-basic-config-column-2">

                             <div class="vf-basic-config-sub-row">
                                <div><?php echo VF_LANGUAGE_LIBRARY['basicConfig']['selectLanguage']; ?>
                                    <select  id="vfLanguage" name="<?php echo self:: BASIC_CONFIG_OPTION_NAMES['selected_language']; ?>">
                                        <option value="English" <?php selected('English', $this->vf_selected_language);?>>English</option>
                                        <option value="German"  <?php selected('German', $this->vf_selected_language);?> >German</option>                                        
                                        <option value="Portuguese"  <?php selected('Portuguese', $this->vf_selected_language);?> >Portuguese</option>
                                        <option value="Chinese"  <?php selected('Chinese', $this->vf_selected_language);?> >Chinese</option>
                                        <option value="French"  <?php selected('French', $this->vf_selected_language);?> >French</option>
                                        <option value="Japanese"  <?php selected('Japanese', $this->vf_selected_language);?> >Japanese</option>
                                        <option value="Korean"  <?php selected('Korean', $this->vf_selected_language);?> >Korean</option>
                                        <option value="Spanish"  <?php selected('Spanish', $this->vf_selected_language);?> >Spanish</option>
                                    </select>
                                </div>                            
                            </div>

                            <div class="vf-basic-config-sub-row">
                                <div class="vf-basic-config-attached-label-column"><?php echo VF_LANGUAGE_LIBRARY['basicConfig']['licenseKey']; ?></div>
                                <div class="vf-basic-config-attached-input-column">
                                    <input 
                                    type="text" 
                                    name="<?php echo self::BASIC_CONFIG_OPTION_NAMES['license_key']; ?>" 
                                    id="vfavigationLicenseKey" 
                                    placeholder="<?php echo VF_LANGUAGE_LIBRARY['basicConfig']['copyYourLicenseKey']; ?>" 
                                    value="<?php echo $this->vf_license_key; ?>"/>
                                </div>
                            </div>
                            <div class="vf-basic-config-sub-row">
                                <span class="vf-autotimeout-label">
                                    <input 
                                        class="vf-autotimeout-mic"
                                        type='number' 
                                        name="<?php echo self::BASIC_CONFIG_OPTION_NAMES['mic_listening_timeout']; ?>" 
                                        min="8"
                                        max="20"
                                        step="1"
                                        onKeyup="vfResetTimeoutDefaultValue(this, event)"
                                        onKeydown="vfValidateTimeoutValue(this, event)"
                                        value="<?php echo $this->vf_mic_listening_timeout; ?>"
                                    /> <?php echo VF_LANGUAGE_LIBRARY['basicConfig']['autoTimeoutDuration']; ?></span>
                            </div>

                            <div class="vf-basic-config-sub-row">
                                <label for="vfFloatingMic">
                                    <input 
                                    id="vfFloatingMic"
                                    type='checkbox' 
                                    name="<?php echo self::BASIC_CONFIG_OPTION_NAMES['floating_mic']; ?>" 
                                    value="yes" <?php checked('yes', $this->vf_floating_mic);?> 
                                    > <?php echo VF_LANGUAGE_LIBRARY['basicConfig']['floatingMic']; ?> 
                                </label>
                            </div>
                        </div>

                        <div class="vf-setting-basic-config-column-3 vf-basic-config-sub-row">
                            <?php 
                            $other_attributes = array( 'id' => 'vfavigationBasicConfigSettingsSave' );
                            submit_button( 
                                VF_LANGUAGE_LIBRARY['basicConfig']['saveSettings'], 
                                'primary', 
                                'vf-basic-config-settings-save', 
                                false, 
                                $other_attributes
                            );
                            ?>
                        </div>
                    </div>
                </form>
            </div>
        </div>
<?php
    }

    /**
     * Register and add settings
     */
    public function vf_page_init()
    {
        // Register settings for feilds of 'Basic Configuration' section
        register_setting('vf-basic-config-settings-group', self::BASIC_CONFIG_OPTION_NAMES['license_key']);
        register_setting('vf-basic-config-settings-group', self::BASIC_CONFIG_OPTION_NAMES['mic_listening_timeout']);
        register_setting('vf-basic-config-settings-group', self::BASIC_CONFIG_OPTION_NAMES['selected_language']);
        register_setting('vf-basic-config-settings-group', self::BASIC_CONFIG_OPTION_NAMES['floating_mic']);
    }
}

// check user capabilities and hook into 'init' to initialize 'Voice Forms' settings object
add_action('init', 'initialize_vf_settings_object');

/**
 * Initialize 'Voice Forms' settings object when 'pluggable' files are loaded from '/wp-includes/pluggable'
 * Which contains 'current_user_can' function.
 */
function initialize_vf_settings_object(){
    if ( !current_user_can( 'manage_options' ) ) return;  

    $voice_forms_settings_page = new Voice_Forms_Settings_Page();
}